QuickStrat = LibStub("AceAddon-3.0"):NewAddon("QuickStrat", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("QuickStrat")
local options = {
    name = "QuickStrat",
    handler = QuickStrat,
    type = "group",
    args = {
        msg = {
            type = "input",
            name = L["Message"],
            desc = L["The message to be displayed when you get home."],
            usage = L["<Your message>"],
            get = "GetMessage",
            set = "SetMessage",
        },
        showInChat = {
            type = "toggle",
            name = L["Show in Chat"],
            desc = L["Toggles the display of the message in the chat window."],
            get = "IsShowInChat",
            set = "ToggleShowInChat",
        },
        showOnScreen = {
            type = "toggle",
            name = L["Show on Screen"],
            desc = L["Toggles the display of the message on the screen."],
            get = "IsShowOnScreen",
            set = "ToggleShowOnScreen"
        },
    },
}

local defaults = {
    profile = {
        message = L["Welcome Home!"],
        showInChat = false,
        showOnScreen = true,
		strats = {},
    },
}

local AceGUI = LibStub("AceGUI-3.0")
local textStore

Strat = {};
Strat.__index = Strat;
function Strat:new(zone, name, x, y)
	local self = {};         -- Create a blank table
	setmetatable(self, Strat);
	self.name = name;
	self.zone = zone;
	self.next = 0;
	self.parts = {};
	self.x = x;
	self.y = y;
	
	if not name then
		return nil
	end
	
	if Strat.db[self.zone] and Strat.db[self.zone][self.name] then
		for i,v in ipairs(Strat.db[self.zone][self.name]["parts"]) do
			table.insert(self.parts, v);
		end
	end

	if Strat.customdb[self.zone] and Strat.customdb[self.zone][self.name] then
		for i,v in ipairs(Strat.customdb[self.zone][self.name]["parts"]) do
			table.insert(self.parts, v);
		end
	end
	
	return self;             -- Return the instance
end

function Strat:find(zone, name, x, y)
	if not (Strat.lastFound and Strat.lastFound:matches(zone, name, x, y)) then
		if not name then
			name = Strat:findNameCoords(zone, x, y);
		end
		Strat.lastFound = Strat:new(zone, name, x, y);
	end
	return Strat.lastFound;
end

function Strat:matches(zone, name, x, y)
	if name then
		return self.name == name and self.zone == zone
	else
		return false;
	end
end

function Strat:findNameCoords( zone, x, y )
	if Strat.customdb[zone] then
		for k,v in pairs(Strat.customdb[zone]) do
			if v["x"] and v["y"] then
				if math.max(math.abs(v["x"] - x), math.abs(v["y"] - y)) < 10 then
					return k;
				end
			end
		end
	end
	return nil;
end

function Strat:GetNextPart()
	if #self.parts == 0 then
		return nil;
	end
	self.next = self.next + 1;
	if self.next > #self.parts then
		self.next = 1;
	end
  	return self.parts[self.next];
end

function Strat:AddPart( part )
	if not Strat.customdb[self.zone] then
		Strat.customdb[self.zone] = {}
	end

	if not Strat.customdb[self.zone][self.name] then
		Strat.customdb[self.zone][self.name] = {
				["parts"]={},
				["x"] = self.x,
				["y"] = self.y,
			};
	end
	
	table.insert(Strat.customdb[self.zone][self.name]["parts"], part);
	table.insert(self.parts, part);
end

function QuickStrat:OnInitialize()
    -- Called when the addon is loaded
    self.db = LibStub("AceDB-3.0"):New("QuickStratDB", defaults, "Default")

    LibStub("AceConfig-3.0"):RegisterOptionsTable("QuickStrat", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("QuickStrat", "QuickStrat")
    self:RegisterChatCommand(L["quickstrat"], "ChatCommand")
    self:RegisterChatCommand(L["qs"], "ChatCommand")
	self.currStrat = nil;
	
	Strat.customdb = self.db.profile.strats;
	Strat.db = L["strats"];
end

QuickStrat.channelcommands = {
	s="SAY",
	say="SAY",
	p="PARTY",
	party="PARTY",
}

function QuickStrat:ChatCommand(input)
	local cmd = self:GetArgs(input, 1);

	if cmd == "config" then
		self:DrawUI();
	elseif cmd == "clear" then
		self.db.profile.strats = {};
		Strat.customdb = self.db.profile.strats;
		self:Print("Cleared all custom entries");
	elseif not cmd or QuickStrat.channelcommands[cmd] then
		cmd, name, zone = self:GetArgs(input, 3);
		zone, name, px, py = QuickStrat:ResolveTarget(zone, name);
		
		self:Print("Finding strat for " .. (name or "[nil]") .. " in zone " .. zone .. " (" .. px .. "," .. py .. ")");
		self.currentStrat = Strat:find(zone, name, px, py, self);
		
		p = self.currentStrat:GetNextPart();
		if p == nil then
			self:Print("No strat for " .. (name or "[nil]") .. " in zone " .. zone .. " (" .. px .. "," .. py .. ")");
		else
			self:SendChat(p, QuickStrat.channelcommands[cmd]);
		end

	elseif cmd == "add" then
		cmd, desc, name, zone = self:GetArgs(input, 4);
		zone, name, px, py = QuickStrat:ResolveTarget(zone, name);
		
		if not name then
			self:Print("Must provide a name or be targeting a mob to add a strat");
			return
		end

		self:Print("Finding strat for " .. name .. " in zone " .. zone);
		self.currentStrat = Strat:find(zone, name, px, py, self);
		
		self.currentStrat:AddPart(desc);
		self:Print("Added strat to " .. name .. " in zone " .. zone .. " (" .. px .. "," .. py .. ")");
	else
		self:Print("Command not understood: " .. cmd);
    end
end

function QuickStrat:ResolveTarget( zone, name )
	px, py = GetPlayerMapPosition("player");
	px = px * 100;
	py = py * 100;
	if name then
		if not zone then
			zone = default
		end
	else
		zone = GetZoneText()
		name = GetUnitName("target", false)
	end
	return zone, name, px, py
end

function QuickStrat:SendChat(message, channel)
	for k,v in next, QuickStrat.splitstring(message, '\n') do
		if channel then
			SendChatMessage("[QuickStrat] " .. v, channel);
		else
			self:Print(v);
		end
	end
end

function QuickStrat:DrawUI()
	-- Create the frame container
	local frame = AceGUI:Create("Frame")
	frame:SetTitle("Quick Strat")
	-- frame:SetStatusText("AceGUI-3.0 Example Container Frame")
	frame:SetCallback("OnClose", function(widget, event) AceGUI:Release(widget) end)

	-- Fill Layout - the TabGroup widget will fill the whole frame
	frame:SetLayout("Fill")

	tree = {};
	for zonek,zonev in pairs(Strat.db) do
		zone = {};
		zone["value"] = zonek;
		zone["text"] = zonek;
		zonec = {};
		for bossk,bossv in pairs(zonev) do
			boss = {};
			boss["value"] = bossk;
			boss["text"] = bossk;
			bossc = {};
			for partk,partv in pairs(bossv["parts"]) do
				part = {}
				part["value"] = partk;
				part["text"] = partk;
				table.insert(bossc, part)
			end
			boss["children"] = bossc;
			table.insert(zonec, boss);
		end
		zone["children"] = zonec;
		-- self:Print(zonek);
		-- self:Print(zonev);
		table.insert(tree, zone);
	end
	
	local t = AceGUI:Create("TreeGroup");
  t:SetLayout("Fill")
  t:SetTree(tree);
  frame:AddChild(t)
  
end


function QuickStrat:OnEnable()
    -- Called when the addon is enabled
end

function QuickStrat:OnDisable()
    -- Called when the addon is disabled
end

function QuickStrat:GetMessage(info)
    return self.db.profile.message
end

function QuickStrat:SetMessage(info, newValue)
    self.db.profile.message = newValue
end

function QuickStrat:IsShowInChat(info)
    return self.db.profile.showInChat
end

function QuickStrat:ToggleShowInChat(info, value)
    self.db.profile.showInChat = value
end

function QuickStrat:IsShowOnScreen(info)
    return self.db.profile.showOnScreen
end

function QuickStrat:ToggleShowOnScreen(info, value)
    self.db.profile.showOnScreen = value
end

function QuickStrat:splitstring(sSeparator, nMax, bRegexp)
	assert(sSeparator ~= '')
	assert(nMax == nil or nMax >= 1)

	local aRecord = {}

	if self:len() > 0 then
		local bPlain = not bRegexp
		nMax = nMax or -1

		local nField=1 nStart=1
		local nFirst,nLast = self:find(sSeparator, nStart, bPlain)
		while nFirst and nMax ~= 0 do
			aRecord[nField] = self:sub(nStart, nFirst-1)
			nField = nField+1
			nStart = nLast+1
			nFirst,nLast = self:find(sSeparator, nStart, bPlain)
			nMax = nMax-1
		end
		aRecord[nField] = self:sub(nStart)
	end

	return aRecord
end